#include <stdlib.h>

#include "Star.h"
#include "WorldManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "GraphicsManager.h"

Star::Star() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);
  setType("Star");
  is_solid = false;

  move_slowdown = random()%10;
  move_countdown = move_slowdown;

  altitude = -1;

  // Sets it so stars do not spawn in the top line over the hud
  pos.setX(random()%world_manager.getBoundary().getHorizontal());
  pos.setY(random()% (world_manager.getBoundary().getVertical() - 1) + 1);
}

Star::~Star() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

void Star::draw() {
  GraphicsManager &graph_mgr = GraphicsManager::getInstance();
  graph_mgr.drawCh(pos, STAR_CHAR);
}

// handle event
// return 0 if ignored, else 1
int Star::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) {
    out();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

// if star moved off screen, move back to far right
void Star::out() {
  WorldManager &world_manager = WorldManager::getInstance();
  pos.setX(world_manager.getBoundary().getHorizontal() + random()%20);
  pos.setY(random() % (world_manager.getBoundary().getVertical() - 1) + 1);
  move_slowdown = random()%10;
}

void Star::move() {
  move_countdown--;
  if (move_countdown > 0)
    return;
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.moveObj(this, new_pos);

  return;
}
