// Engine includes
#include "GameManager.h"
#include "LogManager.h"
#include "ResourceManager.h"

// Game includes
#include "Saucer.h"
#include "Hero.h"
#include "Star.h"
#include "GameStart.h"
#include "Display.h"

#include <Position.h>

// Function prototypes
void loadResources();
void populateWorld();
int display_string(string a_str, Position world_pos);

int main(int argc, char *argv[]) {
  LogManager &log_manager = LogManager::getInstance();

  // Start up Game Manager
  GameManager &game_manager = GameManager::getInstance();
  if (game_manager.startUp())  {
    log_manager.writeLog("Error starting game manager!");
    game_manager.shutDown();
  }

  // Load game resources
  loadResources();

  // Setup some objects
  populateWorld();

  // Run the game (this blocks until the game loop is over)
  game_manager.run();

  // Shut everything down
  game_manager.shutDown();
}

// load resources (sprites, here)
void loadResources() {
  ResourceManager &resource_manager = ResourceManager::getInstance();
  resource_manager.loadSprite("sprites/saucer-spr.txt", "saucer");
  resource_manager.loadSprite("sprites/ship-spr.txt", "ship");
  resource_manager.loadSprite("sprites/bullet-spr.txt", "bullet");
  resource_manager.loadSprite("sprites/explosion-spr.txt", "explosion");
  resource_manager.loadSprite("sprites/gamestart-spr.txt", "gamestart");
  resource_manager.loadSprite("sprites/gameover-spr.txt", "gameover");
}

// populate world with some objects
void populateWorld() {
  // create the GameStart object
  string temp = "gamestart";
  new GameStart(temp);
}

